"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const xlib = tslib_1.__importStar(require("xlib"));
var __ = xlib.lolo;
/** This is a specialized class used for commercial SAAS projects that have a free usage tier, and need to protect against abusive free-tier users.

tracks a given resource, ensuring that multiple (free tier) apiKeys/ip's are not accessing it to get around free usage limits.

once the abuse threshhold is passed, future requests to the resource are blacklisted.*/
class UseLimitFraudCheck {
    constructor(options) {
        this.options = options;
        /** stores all resources and their associated tracking data */
        this._storage = new Map();
    }
    /** private helper that cleans up an entry in our _storage Map. */
    _doHousekeeping(resource) {
        let resourceInfo = this._storage.get(resource);
        if (resourceInfo == null) {
            return;
        }
        let now = Date.now();
        //clean expired tracks
        let windowStart = __.utc().minus(this.options.sampleWindowMs);
        for (const [apiKey, lastUse] of resourceInfo.requests) {
            if (lastUse.valueOf() < windowStart.valueOf()) {
                //old, remove as it's expired
                resourceInfo.requests.delete(apiKey);
            }
        }
        //clean up expires maybe
        if (resourceInfo.blacklistExpires != null && resourceInfo.blacklistExpires.valueOf() < now) {
            resourceInfo.blacklistExpires = null;
        }
        //delete from storage if not blacklisted and empty
        if (resourceInfo.requests.size === 0 && resourceInfo.blacklistExpires == null) {
            this._storage.delete(resource);
        }
    }
    isBlacklisted(apiKey, resource) {
        const _storage = this._storage;
        //get proper domain bucket
        let resourceInfo;
        if (_storage.has(resource)) {
            resourceInfo = _storage.get(resource);
        }
        else {
            resourceInfo = { requests: new Map() };
            _storage.set(resource, resourceInfo);
        }
        //set cleanup 5 minutes from now.
        if (resourceInfo.cleanupTimeoutHandle != null) {
            //there was already a cleanup scheduled.    stop it (rescheduled on the line after)
            clearTimeout(resourceInfo.cleanupTimeoutHandle);
        }
        resourceInfo.cleanupTimeoutHandle = setTimeout(() => {
            //do a cleanup pass for this domain
            this._doHousekeeping(resource);
        }, this.options.sampleWindowMs.valueOf() + 500); //500ms after sample window ends
        //if blacklist is in effect, lets just stop now.
        if (resourceInfo.blacklistExpires != null) {
            if (resourceInfo.blacklistExpires.valueOf() > Date.now()) {
                return true;
            }
            else {
                //expired, clear it.
                resourceInfo.blacklistExpires = null;
            }
        }
        //track
        resourceInfo.requests.set(apiKey, new Date());
        //count all calls to the domain in last 5 min
        let shouldBlacklist = false;
        let requestsInSampleWindow = 0;
        let sampleWindowStart = __.utc().minus(this.options.sampleWindowMs);
        resourceInfo.requests.forEach((lastUse, otherApiKey) => {
            if (lastUse.valueOf() >= sampleWindowStart.valueOf()) {
                requestsInSampleWindow++;
            }
            else {
                //old, remove as it's expired
                resourceInfo.requests.delete(otherApiKey);
            }
        });
        //determine if request should be blacklisted
        if (requestsInSampleWindow >= this.options.blacklistImmediateUses) {
            //too many abuses, blacklist ASAP
            shouldBlacklist = true;
        }
        else if (resourceInfo.blacklistStarts == null && requestsInSampleWindow >= this.options.maxUses) {
            //abused and not yet scheduled for blacklist. blacklist in 5
            resourceInfo.blacklistStarts = __.utc().plus(this.options.obfuscateBlacklistDelayMs).toJSDate();
        }
        //if we have a delayed blacklist, check if it's time to start
        if (resourceInfo.blacklistStarts != null && resourceInfo.blacklistStarts.valueOf() >= Date.now()) {
            shouldBlacklist = true;
        }
        if (shouldBlacklist === true) {
            resourceInfo.blacklistStarts = null;
            resourceInfo.blacklistExpires = __.utc().plus(this.options.blacklistDuration).toJSDate();
        }
        //check if blacklisted
        if (resourceInfo.blacklistExpires != null) {
            if (resourceInfo.blacklistExpires.valueOf() > Date.now()) {
                return true;
            }
        }
        return false;
    }
}
//# sourceMappingURL=saas.js.map